from pyspedas.utilities.dailynames import dailynames
from pyspedas.utilities.download import download
from pyspedas.analysis.time_clip import time_clip as tclip
from pytplot import netcdf_to_tplot

from .config import CONFIG


def loadr(trange=['2023-01-01', '2032-01-02'],
          probe='16',
          instrument='mag',
          datatype='1min',
          prefix='',
          suffix='',
          downloadonly=False,
          no_update=False,
          time_clip=False):
    """
    This function loads GOES-R L2 data (GOES-16, GOES-17, GOES-18)

    Parameters
    ----------
        trange : list of str
            time range of interest [starttime, endtime] with the format
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        probe: str/int or list of strs/ints
            GOES spacecraft #, e.g., probe=16

        instrument: str
            name of the instrument (euvs, xrs, mag, mpsh, sgps)

        datatype: str
            Data resolution, default is '1min'
            Valid options: low (avg), hi (full), and various other options depending on instrument

        prefix: str
            The tplot variable names will be given this prefix.
            By default, no prefix is added.
            If 'probename' then the name will be used, for example 'g16_'.

        suffix: str
            The tplot variable names will be given this suffix.
            By default, no suffix is added.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into
            tplot variables

        no_update: bool
            If set, only load data from your local cache

        time_clip: bool
            Time clip the variables to exactly the range specified in the trange keyword

    Returns
    -------
        List of tplot variables created. Or list of filenames downloaded.

    Notes
    -----
    Information: https://www.ngdc.noaa.gov/stp/satellite/goes-r.html
    Data: https://data.ngdc.noaa.gov/platforms/solar-space-observing-satellites/goes/
    Path: goesNN/l2/data/instrument/YYYY/MM/file.nc
    Time variable: 'time', seconds since 2000-01-01 12:00:00

    GOES-EAST (GOES-16, 2017-)
    GOES-WEST (GOES-17, 2018-2022; GOES-18, 2023-)

    Instruments:    euvs (hi, full, 1min, euvs-l2-avg1m_science/2021/05/sci_euvs-l2-avg1m_g16_d20210530_v1-0-3.nc)
                    euvs (low, avg, 1day, euvs-l2-avg1d_science/2021/06/sci_euvs-l2-avg1d_g16_d20210630_v1-0-3.nc)
                    xrs  (hi, full, 1sec, xrsf-l2-flx1s_science/2022/08/sci_xrsf-l2-flx1s_g16_d20220830_v2-1-0.nc)
                    xrs  (low, avg, 1min, xrsf-l2-avg1m_science/2021/06/sci_xrsf-l2-avg1m_g16_d20210630_v2-1-0.nc)
                    mag  (hi, full, 0.1sec, magn-l2-hires/2021/06/dn_magn-l2-hires_g16_d20210629_v1-0-1.nc)
                    mag  (low, avg, 1min, magn-l2-avg1m/2022/12/dn_magn-l2-avg1m_g16_d20221230_v2-0-2.nc)
                    mpsh (hi, full, 1min, mpsh-l2-avg1m/2022/12/sci_mpsh-l2-avg1m_g16_d20221230_v2-0-0.nc)
                    mpsh (low, avg, 5min, mpsh-l2-avg5m/2022/12/sci_mpsh-l2-avg5m_g16_d20221230_v2-0-0.nc)
                    sgps (hi, full, 1min, sgps-l2-avg1m/2022/12/sci_sgps-l2-avg1m_g17_d20221230_v3-0-0.nc)
                    sgps (low, avg, 5min, sgps-l2-avg5m/2022/12/sci_sgps-l2-avg5m_g17_d20221230_v3-0-0.nc)

    EXIS (Extreme Ultraviolet and X-ray Sensors), EUVS and XRS
    EUVS: Spectral line irradiances, the Mg II index, and proxy spectra from the EXIS Extreme Ultraviolet Sensor (EUVS)
    EUVS: Daily averages of spectral line irradiances, the Mg II index, and proxy spectra
    XRS: 1-minute averages of XRS measurements
    XRS: High cadence measurements from the EXIS X-Ray Sensor (XRS)
    MAG (Magnetometer)
    MAG: Full resolution magnetic field readings in different coordinate systems
    MAG: Averages of 10 Hz magnetometer field readings
    SEISS (Space Environment In Situ Suite): 1-min and 5-min averages for the Magnetospheric Particle Sensors (MPS-HI and MPS-LO)
            and for the Solar and Galactic Proton Sensor (SGPS)

    Wrappers:
        pyspedas.goes.euvs
        pyspedas.goes.xrs
        pyspedas.goes.mag
        pyspedas.goes.mpsh
        pyspedas.goes.sgps

    Example
    -------
        from pyspedas.goes import load
        trange = ['2023-01-01', '2023-01-02']
        load(trange=trange, probe='16', instrument='mag', datatype='1min', time_clip=True)

    """
    goes_path_dir = 'https://data.ngdc.noaa.gov/platforms/solar-space-observing-satellites/goes/'
    time_var = 'time'  # name of the time variable in the netcdf files
    out_files = []
    tvars = []

    if not isinstance(probe, list):
        probe = [probe]

    for prb in probe:
        remote_path = 'goes' + str(prb) + '/l2/data/'

        if instrument == 'euvs':
            if datatype in ['full', 'hi', '1min', 'avg1m']:  # high resolution 1 min
                pathformat = [remote_path + 'euvs-l2-avg1m_science/%Y/%m/sci_euvs-l2-avg1m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
            else:  # low resolution 1 day, smaller files
                pathformat = [remote_path + 'euvs-l2-avg1d_science/%Y/%m/sci_euvs-l2-avg1d_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
        elif instrument == 'xrs':
            if datatype in ['full', 'hi', '1sec', 'flx1s']:  # high resolution 1 sec
                pathformat = [remote_path + 'xrsf-l2-flx1s_science/%Y/%m/sci_xrsf-l2-flx1s_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
            else:  # low resolution 1 min, smaller files
                pathformat = [remote_path + 'xrsf-l2-avg1m_science/%Y/%m/sci_xrsf-l2-avg1m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
        elif instrument == 'mag':
            if datatype in ['full', 'hi', '0.1sec', 'hires']:  # high resolution 0.1 sec
                pathformat = [remote_path + 'magn-l2-hires/%Y/%m/dn_magn-l2-hires_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
            else:  # low resolution 1 min, smaller files
                pathformat = [remote_path + 'magn-l2-avg1m/%Y/%m/dn_magn-l2-avg1m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
        elif instrument == 'mpsh':
            time_var = 'L2_SciData_TimeStamp'
            if datatype in ['full', 'hi', '1min', 'avg1m', '1m']:  # high resolution 1 min
                pathformat = [remote_path + 'mpsh-l2-avg1m/%Y/%m/sci_mpsh-l2-avg1m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
            else:  # low resolution 5 min, smaller files
                pathformat = [remote_path + 'mpsh-l2-avg5m/%Y/%m/sci_mpsh-l2-avg5m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
        elif instrument == 'sgps':
            if datatype in ['full', 'hi', '1min', 'avg1m', '1m']:  # high resolution 1 min
                pathformat = [remote_path + 'sgps-l2-avg1m/%Y/%m/sci_sgps-l2-avg1m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']
            else:  # low resolution 5 min, smaller files
                pathformat = [remote_path + 'sgps-l2-avg5m/%Y/%m/sci_sgps-l2-avg5m_g' + str(prb) + '_d%Y%m%d_v?-?-?.nc']

        # find the full remote path names using the trange
        if not isinstance(pathformat, list):
            pathformat = [pathformat]

        remote_names = []
        for path in pathformat:
            remote_names.extend(dailynames(file_format=path, trange=trange))

        files = download(remote_file=remote_names, remote_path=goes_path_dir, local_path=CONFIG['local_data_dir'], no_download=no_update)

        if files is not None:
            for file in files:
                out_files.append(file)

        tvars_local = []
        if len(files) > 0 and downloadonly is False:
            if prefix == 'probename':
                prefix_local = 'g' + str(prb) + '_'
            else:
                prefix_local = prefix
            tvars_local = netcdf_to_tplot(files, prefix=prefix_local, suffix=suffix, merge=True, time=time_var)

        if len(tvars_local):
            tvars.extend(tvars_local)

    if downloadonly:
        out_files = sorted(out_files)
        return out_files

    if time_clip:
        for new_var in tvars:
            tclip(new_var, trange[0], trange[1], suffix='')

    return tvars


def load(trange=['2013-11-05', '2013-11-06'],
         probe='15',
         instrument='fgm',
         datatype='1min',
         prefix='',
         suffix='',
         downloadonly=False,
         no_update=False,
         time_clip=False):
    """
    This function loads GOES L2 data

    Parameters
    ----------
        trange : list of str
            time range of interest [starttime, endtime] with the format
            'YYYY-MM-DD','YYYY-MM-DD'] or to specify more or less than a day
            ['YYYY-MM-DD/hh:mm:ss','YYYY-MM-DD/hh:mm:ss']

        probe: str/int or list of strs/ints
            GOES spacecraft #, e.g., probe=15

        instrument: str
            name of the instrument
            (for GOES 8-15: fgm, eps, epead, maged, magpd, hepad, xrs)
            (for GOES-R 16-18: euvs, xrs, mag, mpsh, sgps)

        datatype: str
            Data type; usually instrument resolution, depends on the instrument
            Default is 1min
            (valid for GOES 8-15: hi, low, full, avg, 1min, 5min)
            (valid for GOES-R 16-18: hi, low, full, avg, and other options)

        prefix: str
            The tplot variable names will be given this prefix.
            By default, no prefix is added.
            If 'probename' then the name will be used, for example g16.

        suffix: str
            The tplot variable names will be given this suffix.
            By default, no suffix is added.

        downloadonly: bool
            Set this flag to download the CDF files, but not load them into
            tplot variables

        no_update: bool
            If set, only load data from your local cache

        time_clip: bool
            Time clip the variables to exactly the range specified in the trange keyword

    Returns
    -------
        List of tplot variables created. Or list of filenames downloaded.

    Notes
    -----
    This function loads data from the GOES mission; this function is not meant
    to be called directly; instead, see the wrappers:
        pyspedas.goes.fgm
        pyspedas.goes.eps
        pyspedas.goes.epead
        pyspedas.goes.maged
        pyspedas.goes.magpd
        pyspedas.goes.hepad
        pyspedas.goes.xrs

    """

    if not isinstance(probe, list):
        probe = [probe]

    probe_r = []  # GOES-R probes
    probe_s = []  # GOES probes
    out_files_r = []  # GOES-R files to download
    out_files = []  # GOES files to download
    tvars_r = []  # GOES-R list of variables created
    tvars = []  # all variables created

    # Find if we need to call the GOES-R load function for some probes
    for prb in probe:
        if int(prb) > 15:
            probe_r.append(str(prb))
        else:
            probe_s.append(str(prb))

    if len(probe_r):
        tvars_r = loadr(trange=trange,
                        probe=probe_r,
                        instrument=instrument,
                        datatype=datatype,
                        prefix=prefix,
                        suffix=suffix,
                        downloadonly=downloadonly,
                        no_update=no_update,
                        time_clip=time_clip)
        if downloadonly:
            out_files_r = tvars_r

    # Continue with loading GOES (1-15) data
    for prb in probe_s:
        avg_path = 'avg/%Y/%m/goes' + str(prb) + '/netcdf/' + 'g' + str(prb)
        full_path = 'full/%Y/%m/goes' + str(prb) + '/netcdf/' + 'g' + str(prb)

        if instrument == 'fgm':
            if datatype == '512ms' or datatype == 'full':  # full, unaveraged data
                pathformat = full_path + '_magneto_512ms_%Y%m%d_%Y%m%d.nc'
            elif datatype == '5min':  # 5 min averages
                pathformat = avg_path + '_magneto_5m_%Y%m01_%Y%m??.nc'
            else:  # 1 min averages, goes13, goes15 only contain 1m averages
                pathformat = avg_path + '_magneto_1m_%Y%m01_%Y%m??.nc'
        elif instrument == 'eps':
            # energetic particle sensor -- only valid for GOES-08 through GOES-12, only averaged data available
            if datatype == '1min' or datatype == 'full':
                pathformat = avg_path + '_eps_1m_%Y%m01_%Y%m??.nc'
            else:  # 'low' or 5min
                pathformat = avg_path + '_eps_5m_%Y%m01_%Y%m??.nc'
        elif instrument == 'epead':
            # electron, proton, alpha detector -- only valid on GOES-13, 14, 15
            if datatype == '1min':
                pathformat = ['_epead_e13ew_1m_%Y%m01_%Y%m??.nc',
                              '_epead_p17ew_1m_%Y%m01_%Y%m??.c',
                              '_epead_a16ew_1m_%Y%m01_%Y%m??.nc']
                pathformat = [avg_path + s for s in pathformat]
            elif datatype == '5min' or datatype == 'low':
                pathformat = ['_epead_e13ew_5m_%Y%m01_%Y%m??.nc',
                              '_epead_p17ew_5m_%Y%m01_%Y%m??.c',
                              '_epead_a16ew_5m_%Y%m01_%Y%m??.nc']
                pathformat = [avg_path + s for s in pathformat]
            else:  # full
                pathformat = ['_epead_a16e_32s_%Y%m%d_%Y%m%d.nc',
                              '_epead_a16w_32s_%Y%m%d_%Y%m%d.nc',
                              '_epead_e1ew_4s_%Y%m%d_%Y%m%d.nc',
                              '_epead_e2ew_16s_%Y%m%d_%Y%m%d.nc',
                              '_epead_e3ew_16s_%Y%m%d_%Y%m%d.nc',
                              '_epead_p1ew_8s_%Y%m%d_%Y%m%d.nc',
                              '_epead_p27e_32s_%Y%m%d_%Y%m%d.nc',
                              '_epead_p27w_32s_%Y%m%d_%Y%m%d.nc']
                pathformat = [full_path + s for s in pathformat]
        elif instrument == 'maged':
            # magnetospheric electron detector -- only valid on GOES 13, 14, 15
            if datatype == '1min':
                pathformat = avg_path + '_maged_19me15_1m_%Y%m01_%Y%m??.nc'
            elif datatype == '5min' or datatype == 'low':
                pathformat = avg_path + '_maged_19me15_5m_%Y%m01_%Y%m??.nc'
            else:  # full
                channels = ['me1', 'me2', 'me3', 'me4', 'me5']
                resolution = ['2', '2', '4', '16', '32']
                pathformat = []
                for idx, channel in enumerate(channels):
                    pathformat.append('_maged_19' + channel + '_' + resolution[idx] + 's_%Y%m%d_%Y%m%d.nc')
                pathformat = [full_path + s for s in pathformat]
        elif instrument == 'magpd':
            # magnetospheric proton detector -- only valid on GOES 13, 14, 15
            if datatype == '1min' or datatype == 'low':
                pathformat = avg_path + '_magpd_19mp15_1m_%Y%m01_%Y%m??.nc'
            else:  # full
                channels = ['mp1', 'mp2', 'mp3', 'mp4', 'mp5']
                resolution = ['16', '16', '16', '32', '32']
                pathformat = []
                for idx, channel in enumerate(channels):
                    pathformat.append('_magpd_19' + channel + '_'+resolution[idx] + 's_%Y%m%d_%Y%m%d.nc')
                pathformat = [full_path + s for s in pathformat]
        elif instrument == 'hepad':
            # high energy proton and alpha detector -- valid for GOES 08-15
            if datatype == '1min':
                pathformat = ['_hepad_ap_1m_%Y%m01_%Y%m??.nc',
                              '_hepad_s15_1m_%Y%m01_%Y%m??.nc']
                pathformat = [avg_path + s for s in pathformat]
            elif datatype == '5min' or datatype == 'low':
                pathformat = ['_hepad_ap_5m_%Y%m01_%Y%m??.nc',
                              '_hepad_s15_5m_%Y%m01_%Y%m??.nc']
                pathformat = [avg_path + s for s in pathformat]
            else:
                pathformat = ['_hepad_ap_32s_%Y%m%d_%Y%m%d.nc',
                              '_hepad_s15_4s_%Y%m%d_%Y%m%d.nc']
                pathformat = [full_path + s for s in pathformat]
        elif instrument == 'xrs':
            # x-ray sensor -- valid for GOES 08-15
            if datatype == '1min':
                pathformat = avg_path + '_xrs_1m_%Y%m01_%Y%m??.nc'
            elif datatype == '5min' or datatype == 'low':
                pathformat = avg_path + '_xrs_5m_%Y%m01_%Y%m??.nc'
            else:
                pathformat = ['_xrs_2s_%Y%m%d_%Y%m%d.nc',
                              '_xrs_3s_%Y%m%d_%Y%m%d.nc']
                pathformat = [full_path + s for s in pathformat]

        # find the full remote path names using the trange
        if not isinstance(pathformat, list):
            pathformat = [pathformat]

        remote_names = []
        for path in pathformat:
            remote_names.extend(dailynames(file_format=path, trange=trange))

        files = download(remote_file=remote_names, remote_path=CONFIG['remote_data_dir'], local_path=CONFIG['local_data_dir'], no_download=no_update)
        if files is not None:
            for file in files:
                out_files.append(file)

        tvars_local = []
        if len(files) > 0 and downloadonly is False:
            if prefix == 'probename':
                prefix_local = 'g' + str(prb) + '_'
            else:
                prefix_local = prefix
            tvars_local = netcdf_to_tplot(files, prefix=prefix_local, suffix=suffix, merge=True, time='time_tag')

        if len(tvars_local) > 0:
            tvars.extend(tvars_local)

    if downloadonly:
        out_files.extend(out_files_r)  # append GOES-R filenames
        out_files = sorted(out_files)
        return out_files

    if time_clip:
        for new_var in tvars:
            tclip(new_var, trange[0], trange[1], suffix='')

    if len(tvars_r):
        tvars.extend(tvars_r)  # append GOES-R variables

    return tvars



import numpy as np

from astropy.coordinates import cartesian_to_spherical


def get_lepi_flux_angle_in_sga(looking_dir=False):

    """
    ;; Here, phi is the angles of the flux directions in the SGA Y-Z plane
    ;; as defined to be angles from the +Y axis increasing toward the +Z axis. 
    ;; Please see Asamura+, EPS, 2018 for the details.    
    """

    phi_array = np.array( 
        [ 101.25, 123.75, 146.25, 168.75, 191.25, 213.75, 236.25, 258.75,
          281.25, 303.75, 326.25, 348.75, 33.75, 56.25, 78.75 ]
    )

    #;; Inclination of apertures = 0.0 deg
    theta_array = np.full(shape=(15), fill_value=0.)

    """
    ;; unit vector of looking direction 
    ;;   e_array is a 2-D array [azch, 3(x,y,z)] in SGA/SGI 
    """
    dtor = np.pi / 180.

    e_array = np.array([
     1. * np.sin(theta_array*dtor),
     1. * np.cos(theta_array*dtor) * np.cos(phi_array*dtor),
     1. * np.cos(theta_array*dtor) * np.sin(phi_array*dtor)
     ])

    if looking_dir:
        e_array *= -1.  # ;;Flip the directions to be looking dirs

    spherical_data = cartesian_to_spherical(
        x=e_array[0],
        y=e_array[1],
        z=e_array[2]
    )

    rtod = 180. / np.pi
    elev_array = spherical_data[1].value * rtod
    phi_array = spherical_data[2].value * rtod

    anglarr = np.zeros(shape=(2, 3, 15))  # ;[ elev/phi, min/cnt/max, apd_no ]

    anglarr[0] = elev_array
    anglarr[1] = phi_array

    return anglarr



import os
import unittest
from pyspedas.utilities.data_exists import data_exists
import pyspedas
from pytplot import del_data
from pyspedas.rbsp.rbspice_lib.rbsp_rbspice_pad import rbsp_rbspice_pad


class LoadTestCases(unittest.TestCase):
    def tearDown(self):
        del_data('*')

    def test_downloadonly(self):
        files = pyspedas.rbsp.efw(trange=['2015-11-3', '2015-11-4'], level='l3', downloadonly=True)
        self.assertTrue(os.path.exists(files[0]))

    def test_notplot(self):
        data = pyspedas.rbsp.efw(trange=['2015-11-6', '2015-11-7'], level='l3', notplot=True)
        self.assertTrue('density' in data.keys())
        self.assertTrue('efield_in_inertial_frame_spinfit_mgse' in data.keys())
        self.assertTrue('x' in data['density'].keys())
        self.assertTrue('y' in data['density'].keys())

    def test_load_emfisis_data(self):
        emfisis_vars = pyspedas.rbsp.emfisis(trange=['2018-11-5', '2018-11-6'], datatype='magnetometer', level='l3', time_clip=True)
        self.assertTrue(data_exists('Mag'))
        emfisis_vars = pyspedas.rbsp.emfisis(trange=['2018-11-5', '2018-11-6'], datatype='magnetometer', level='l2')
        wfr_vars = pyspedas.rbsp.emfisis(trange=['2018-11-5', '2018-11-6'], level='l2', datatype='wfr')
        hfr_vars = pyspedas.rbsp.emfisis(trange=['2018-11-5', '2018-11-6'], level='l2', datatype='hfr')
        self.assertTrue(data_exists('Mag'))
        # WFR waveform data
        self.assertTrue(data_exists('BuSamples'))
        self.assertTrue(data_exists('BvSamples'))
        self.assertTrue(data_exists('BwSamples'))
        self.assertTrue(data_exists('EuSamples'))
        self.assertTrue(data_exists('EvSamples'))
        self.assertTrue(data_exists('EwSamples'))
        # HFR waveform data
        self.assertTrue(data_exists('HFRsamples'))
        # L4 density
        dens = pyspedas.rbsp.emfisis(trange=['2018-11-5', '2018-11-6'], datatype='density', level='l4')
        self.assertTrue(data_exists('density'))

    def test_load_efw_data(self):
        efw_vars = pyspedas.rbsp.efw(trange=['2015-11-5', '2015-11-6'], level='l2')
        self.assertTrue(data_exists('spec64_e12ac'))
        efw_vars = pyspedas.rbsp.efw(trange=['2015-11-5', '2015-11-6'], level='l3')
        self.assertTrue(data_exists('density'))

    def test_load_rbspice_download(self):
        files = pyspedas.rbsp.rbspice(downloadonly=True, trange=['2018-11-5', '2018-11-6'], datatype='tofxeh', level='l3')
        self.assertTrue(isinstance(files, list))

    def test_load_rbspice_data(self):
        data = pyspedas.rbsp.rbspice(trange=['2018-11-5', '2018-11-6'], datatype='TOFxEH', level='l3')
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEH_proton_omni_spin'))
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEH_proton_omni'))
        rbsp_rbspice_pad(probe='a', datatype='TOFxEH', level='l3')
        rbsp_rbspice_pad(probe='a', datatype='TOFxEH', level='l3', scopes=[0, 1, 2, 3])
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEH_proton_omni_0-1000keV_pad'))
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEH_proton_omni_0-1000keV_pad_spin'))
        data = pyspedas.rbsp.rbspice(trange=['2018-11-5', '2018-11-6'], datatype='TOFxPHHHELT')
        rbsp_rbspice_pad(probe='a', datatype='TOFxPHHHELT', level='l3')
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxPHHHELT_oxygen_omni_spin'))
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxPHHHELT_oxygen_omni_0-1000keV_pad_spin'))
        data = pyspedas.rbsp.rbspice(trange=['2018-11-5', '2018-11-6'], datatype='TOFxEnonH')
        rbsp_rbspice_pad(probe='a', datatype='TOFxEnonH', level='l3')
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEnonH_oxygen_omni_spin'))
        self.assertTrue(data_exists('rbspa_rbspice_l3_TOFxEnonH_oxygen_omni_0-1000keV_pad_spin'))

    def test_load_mageis_data(self):
        mageis_vars = pyspedas.rbsp.mageis(trange=['2018-11-5', '2018-11-6'], level='l3', rel='rel04')
        self.assertTrue(data_exists('I'))

    def test_load_hope_data(self):
        hope_vars = pyspedas.rbsp.hope(trange=['2018-11-5', '2018-11-6'], datatype='moments', level='l3', rel='rel04')
        self.assertTrue(data_exists('Ion_density'))
        hope_vars = pyspedas.rbsp.hope(trange=['2018-11-5', '2018-11-6'], datatype='pitchangle', level='l3')
        self.assertTrue(data_exists('FEDO'))
        hope_vars = pyspedas.rbsp.hope(trange=['2018-11-5', '2018-11-6'], datatype='spinaverage', level='l2')
        self.assertTrue(data_exists('I_Ele'))

    def test_load_rep_data(self):
        rept_vars = pyspedas.rbsp.rept(trange=['2018-11-4', '2018-11-5'], level='l2', rel='rel03')
        self.assertTrue(data_exists('FESA'))
        self.assertTrue(data_exists('FEDU'))
        self.assertTrue(data_exists('FPSA'))
        self.assertTrue(data_exists('FPDU'))

    def test_load_rps1min_data(self):
        rps_vars = pyspedas.rbsp.rps()
        self.assertTrue(data_exists('DOSE2_RATE'))
        rps_vars = pyspedas.rbsp.rps(datatype='rps')
        self.assertTrue(data_exists('FPDU_Energy'))


if __name__ == '__main__':
    unittest.main()




import numpy as np

from astropy.coordinates import cartesian_to_spherical

def get_sphcntr(latarr, lonarr):
    if latarr.size != lonarr.size:
        print('get_sphcntr: Array size does not match!')
        return np.array([np.nan, np.nan])
    deg_to_rad = np.pi / 180.
    phiarr = lonarr * deg_to_rad
    thearr = (90. - latarr) * deg_to_rad
    x_array = np.sin(thearr) * np.cos(phiarr)
    y_array = np.sin(thearr) * np.sin(phiarr)
    z_array = np.cos(thearr)
    sum_x = np.nansum(x_array)
    sum_y = np.nansum(y_array)
    sum_z = np.nansum(z_array)
    r_value, theta_value, phi_value = cartesian_to_spherical(sum_x, sum_y, sum_z)
    return [theta_value.value / deg_to_rad , phi_value.value / deg_to_rad]




from .satellite.erg.hep.hep import hep
from .satellite.erg.lepe.lepe import lepe
from .satellite.erg.lepi.lepi import lepi
from .satellite.erg.mepe.mepe import mepe
from .satellite.erg.mepi.mepi_nml import mepi_nml
from .satellite.erg.mepi.mepi_tof import mepi_tof
from .satellite.erg.mgf.mgf import mgf
from .satellite.erg.orb.orb import orb
from .satellite.erg.pwe.pwe_efd import pwe_efd
from .satellite.erg.pwe.pwe_hfa import pwe_hfa
from .satellite.erg.pwe.pwe_ofa import pwe_ofa
from .satellite.erg.pwe.pwe_wfc import pwe_wfc
from .satellite.erg.xep.xep import xep
from .satellite.erg.particle.erg_xep_part_products import erg_xep_part_products
from .satellite.erg.particle.erg_hep_part_products import erg_hep_part_products
from .satellite.erg.particle.erg_mep_part_products import erg_mep_part_products
from .satellite.erg.particle.erg_lep_part_products import erg_lep_part_products

from .ground.camera.camera_omti_asi import camera_omti_asi
from .ground.geomag.gmag_isee_fluxgate import gmag_isee_fluxgate
from .ground.geomag.gmag_isee_induction import gmag_isee_induction
from .ground.geomag.gmag_stel_fluxgate import gmag_stel_fluxgate
from .ground.geomag.gmag_stel_induction import gmag_stel_induction
from .ground.geomag.gmag_magdas_1sec import gmag_magdas_1sec
from .ground.geomag.gmag_mm210 import gmag_mm210
from .ground.radar.superdarn.sd_fit import sd_fit
from .ground.riometer.isee_brio import isee_brio
from .ground.vlf.isee_vlf import isee_vlf



import unittest
from datetime import datetime, timezone

from pyspedas.utilities.time_string import time_string, time_datetime, time_string_one
from pyspedas.utilities.time_double import time_float_one, time_float, time_double


class TimeTestCases(unittest.TestCase):
    def test_time_datetime(self):
        """Test time_datetime function."""
        now = time_datetime()
        self.assertTrue(time_datetime('2015-12-15/00:00') == datetime(2015, 12, 15, 0, 0, tzinfo=timezone.utc))
        self.assertTrue(time_datetime(1450137600.0000000) == datetime(2015, 12, 15, 0, 0, tzinfo=timezone.utc))
        self.assertTrue([time_datetime(1450137600.0000000), time_datetime(1444953600.0000000)] 
            == [datetime(2015, 12, 15, 0, 0, tzinfo=timezone.utc), datetime(2015, 10, 16, 0, 0, tzinfo=timezone.utc)])
        self.assertTrue(time_datetime([1450137600.0000000, 1444953600.0000000])
            == [datetime(2015, 12, 15, 0, 0, tzinfo=timezone.utc), datetime(2015, 10, 16, 0, 0, tzinfo=timezone.utc)])


    def test_time_string(self):
        """Test time_string function."""
        self.assertTrue(time_string(fmt='%Y-%m-%d') == datetime.now().strftime('%Y-%m-%d'))
        self.assertTrue(time_string(1450181243.767, fmt='%Y-%m-%d') == '2015-12-15')
        self.assertTrue(time_string(1450181243.767, fmt='%Y-%m-%d/%H:%M:%S') == '2015-12-15/12:07:23')
        self.assertTrue(time_string(1450181243.767) == '2015-12-15 12:07:23.767000')
        self.assertTrue(time_string([1450181243.767, 1450181263.767]) == ['2015-12-15 12:07:23.767000', '2015-12-15 12:07:43.767000'])

    def test_time_double(self):
        """Test time_double function."""
        self.assertTrue(time_string(time_double(), fmt='%Y-%m-%d') == datetime.now().strftime('%Y-%m-%d'))
        self.assertTrue(time_double('2015-12-15/12:00') == 1450180800.0000000)
        self.assertTrue(time_double('2015-12-15/12') == 1450180800.0000000)
        #self.assertTrue(time_double('2015-12-15/6') == 1450159200.0000000) #this one doesn't work
        self.assertTrue(time_double('2015-12-15/6:00') == 1450159200.0000000)
        self.assertTrue(time_double('2015-12-15/06:00') == 1450159200.0000000)
        self.assertTrue(time_double('2015-12-15') == 1450137600.0000000)
        self.assertTrue(time_double('2015 12 15') == 1450137600.0000000)
        self.assertTrue(time_double('2015-12') == 1448928000.0000000)
        self.assertTrue(time_double('2015') == 1420070400.0000000)
        self.assertTrue(time_double('2015-12-15 12:07:23.767000') == 1450181243.767)
        self.assertTrue(time_double(['2015-12-15 12:07:23.767000', '2015-12-15 12:07:43.767000']) == [1450181243.767, 1450181263.767])

if __name__ == '__main__':
    unittest.main()


import os
import unittest
from pyspedas.utilities.data_exists import data_exists
import pyspedas


class LoadTestCases(unittest.TestCase):
    def test_downloadonly(self):
        files = pyspedas.wind.mfi(trange=['2013-11-5', '2013-11-6'], downloadonly=True)
        self.assertTrue(os.path.exists(files[0]))

    def test_load_3dp_data(self):
        tdp_vars = pyspedas.wind.threedp(trange=['2003-09-5', '2003-09-6'], notplot=True)
        self.assertTrue('N_e_dens_wi_3dp' in tdp_vars)
        tdp_vars = pyspedas.wind.threedp(datatype='3dp_elm2', trange=['2003-09-5', '2003-09-6'], notplot=True)

    def test_load_mfi_data(self):
        mfi_vars = pyspedas.wind.mfi(trange=['2013-11-5', '2013-11-6'], time_clip=True)
        self.assertTrue(data_exists('BGSE'))

    def test_load_swe_data(self):
        swe_vars = pyspedas.wind.swe(trange=['2013-11-5', '2013-11-6'])
        self.assertTrue(data_exists('N_elec'))
        self.assertTrue(data_exists('T_elec'))
        self.assertTrue(data_exists('W_elec'))

    def test_load_waves_data(self):
        swe_vars = pyspedas.wind.waves(trange=['2013-11-5', '2013-11-6'])
        self.assertTrue(data_exists('E_VOLTAGE_RAD1'))
        self.assertTrue(data_exists('E_VOLTAGE_RAD2'))
        self.assertTrue(data_exists('E_VOLTAGE_TNR'))

    def test_load_orbit_data(self):
        orb_vars = pyspedas.wind.orbit(trange=['2013-11-5', '2013-11-6'])
        self.assertTrue(data_exists('GSM_POS'))
        self.assertTrue(data_exists('GSM_VEL'))
        self.assertTrue(data_exists('SUN_VECTOR'))
        self.assertTrue(data_exists('GCI_POS'))
        self.assertTrue(data_exists('GCI_VEL'))

    def test_load_sms_data(self):
        sms_vars = pyspedas.wind.sms()
        self.assertTrue(data_exists('Alpha_vel'))
        self.assertTrue(data_exists('C/O_ratio'))
        self.assertTrue(data_exists('C_ion_temp'))
        self.assertTrue(data_exists('O_ion_temp'))


if __name__ == '__main__':
    unittest.main()



from .gmag_isee_fluxgate import gmag_isee_fluxgate

def gmag_stel_fluxgate(
    trange=['2020-08-01', '2020-08-02'],
    suffix='',
    site='all',
    datatype='all',
    get_support_data=False,
    varformat=None,
    varnames=[],
    downloadonly=False,
    notplot=False,
    no_update=False,
    uname=None,
    passwd=None,
    time_clip=False,
    ror=True
):
    
    return gmag_isee_fluxgate(trange=trange,
                              suffix=suffix,
                              site=site,
                              datatype=datatype,
                              get_support_data=get_support_data,
                              varformat=varformat,
                              varnames=varnames,
                              downloadonly=downloadonly,
                              notplot=notplot,
                              no_update=no_update,
                              uname=uname,
                              passwd=passwd,
                              time_clip=time_clip,
                              ror=ror)